@@ -88,7 +88,7 @@ inline char *get_acpi_file (const char *file) {
 	fd = open(file, O_RDONLY);
 	if (fd == -1) return NULL;
 	end = read(fd, buf, sizeof(buf));
-	buf[end] = '\0';
+	buf[end-1] = '\0';
 	close(fd);
 	return buf;
 }
@@ -168,8 +168,8 @@ int find_items (char *itemname, char infoarray[ACPI_MAXITEM][128],
 	if (dir == NULL)
 		return 0;
 	while ((ent = readdir(dir))) {
-		if (!strncmp(".", ent->d_name, 1) || 
-		    !strncmp("..", ent->d_name, 2))
+		if (!strcmp(".", ent->d_name) || 
+		    !strcmp("..", ent->d_name))
 			continue;
 
 		devices[num_devices]=strdup(ent->d_name);
@@ -245,10 +245,16 @@ int acpi_supported (void) {
 	}
 	closedir(dir);
 	
-	version = get_acpi_value(PROC_ACPI "/info", "ACPI-CA Version:");
+	/* If kernel is 2.6.21 or newer, version is in
+	   /sys/module/acpi/parameters/acpica_version */
+	
+	version = get_acpi_file("/sys/module/acpi/parameters/acpica_version");
 	if (version == NULL) {
-		/* 2.5 kernel acpi */
-		version = get_acpi_value(PROC_ACPI "/info", "version:");
+		version = get_acpi_value(PROC_ACPI "/info", "ACPI-CA Version:");
+		if (version == NULL) {
+			/* 2.5 kernel acpi */
+			version = get_acpi_value(PROC_ACPI "/info", "version:");
+		}
 	}
 	if (version == NULL) {
 		return 0;
